@@ -4,16 +4,15 @@ use std::path::{Path, PathBuf};
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::{Arc, Mutex};
 
-use cargo::core::compiler::unit_graph::UnitDep;
-use cargo::core::compiler::unit_graph::UnitGraph;
-use cargo::core::compiler::Unit;
-use cargo::core::{compiler::Executor, profiles::Profiles};
-use cargo::core::{TargetKind, Workspace};
+use cargo::core::compiler::{unit_graph::UnitDep, unit_graph::UnitGraph, Executor, Unit};
+use cargo::core::profiles::Profiles;
+use cargo::core::{FeatureValue, Package, PackageId, Target, TargetKind, Workspace};
 use cargo::ops::{self, CompileFilter, CompileOptions, FilterRule, LibRule};
 use cargo::util::command_prelude::{ArgMatches, ArgMatchesExt, CompileMode, ProfileChecking};
 use cargo::util::interning::InternedString;
-use cargo::{CliResult, Config};
+use cargo::{CliResult, GlobalContext};
 
+use anyhow::Context as _;
 use cargo_util::paths::{copy, create, create_dir_all, open, read, read_bytes, write};
 use semver::Version;
 
@@ -30,7 +29,7 @@ fn build_include_file(
     root_output: &Path,
     root_path: &Path,
 ) -> anyhow::Result<()> {
-    ws.config()
+    ws.gctx()
         .shell()
         .status("Building", "header file using cbindgen")?;
     let mut header_name = PathBuf::from(name);
@@ -43,7 +42,7 @@ fn build_include_file(
     let warning = config.autogen_warning.unwrap_or_default();
     let version_info = format!(
         "\n#define {0}_MAJOR {1}\n#define {0}_MINOR {2}\n#define {0}_PATCH {3}\n",
-        name.to_uppercase(),
+        name.to_uppercase().replace('-', "_"),
         version.major,
         version.minor,
         version.patch
@@ -65,10 +64,8 @@ fn copy_prebuilt_include_file(
     build_targets: &BuildTargets,
     root_output: &Path,
 ) -> anyhow::Result<()> {
-    let mut shell = ws.config().shell();
+    let mut shell = ws.gctx().shell();
     shell.status("Populating", "uninstalled header directory")?;
-    let path = &format!("PKG_CONFIG_PATH=\"{}\"", root_output.display());
-    shell.verbose(move |s| s.note(path))?;
     for (from, to) in build_targets.extra.include.iter() {
         let to = root_output.join("include").join(to);
         create_dir_all(to.parent().unwrap())?;
@@ -91,7 +88,7 @@ fn build_pc_files(
     root_output: &Path,
     pc: &PkgConfig,
 ) -> anyhow::Result<()> {
-    ws.config().shell().status("Building", "pkg-config files")?;
+    ws.gctx().shell().status("Building", "pkg-config files")?;
     build_pc_file(filename, root_output, pc)?;
     let pc_uninstalled = pc.uninstalled(root_output);
     build_pc_file(
@@ -141,7 +138,7 @@ fn build_def_file(
     let env = &target.env;
 
     if os == "windows" && env == "msvc" {
-        ws.config()
+        ws.gctx()
             .shell()
             .status("Building", ".def file using dumpbin")?;
 
@@ -214,7 +211,7 @@ fn build_implib_file(
     if os == "windows" {
         let arch = &target.arch;
         if env == "gnu" {
-            ws.config()
+            ws.gctx()
                 .shell()
                 .status("Building", "implib using dlltool")?;
 
@@ -243,7 +240,7 @@ fn build_implib_file(
                 Err(anyhow::anyhow!("Command failed {:?}", dlltool_command))
             }
         } else {
-            ws.config().shell().status("Building", "implib using lib")?;
+            ws.gctx().shell().status("Building", "implib using lib")?;
             let target_str = format!("{}-pc-windows-msvc", &target.arch);
             let mut lib = match cc::windows_registry::find(&target_str, "lib.exe") {
                 Some(command) => command,
@@ -400,16 +397,43 @@ pub struct PkgConfigCApiConfig {
     pub strip_include_path_components: usize,
 }
 
+#[derive(Debug)]
+pub enum VersionSuffix {
+    Major,
+    MajorMinor,
+    MajorMinorPatch,
+}
+
 #[derive(Debug)]
 pub struct LibraryCApiConfig {
     pub name: String,
     pub version: Version,
     pub install_subdir: Option<String>,
     pub versioning: bool,
+    pub version_suffix_components: Option<VersionSuffix>,
     pub import_library: bool,
     pub rustflags: Vec<String>,
 }
 
+impl LibraryCApiConfig {
+    pub fn sover(&self) -> String {
+        let major = self.version.major;
+        let minor = self.version.minor;
+        let patch = self.version.patch;
+
+        match self.version_suffix_components {
+            None => match (major, minor, patch) {
+                (0, 0, patch) => format!("0.0.{patch}"),
+                (0, minor, _) => format!("0.{minor}"),
+                (major, _, _) => format!("{major}"),
+            },
+            Some(VersionSuffix::Major) => format!("{major}"),
+            Some(VersionSuffix::MajorMinor) => format!("{major}.{minor}"),
+            Some(VersionSuffix::MajorMinorPatch) => format!("{major}.{minor}.{patch}"),
+        }
+    }
+}
+
 #[derive(Debug, Default)]
 pub struct InstallCApiConfig {
     pub include: Vec<InstallTarget>,
@@ -484,7 +508,10 @@ impl InstallTargetPaths {
     }
 }
 
-fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
+fn load_manifest_capi_config(
+    pkg: &Package,
+    rustc_target: &target::Target,
+) -> anyhow::Result<CApiConfig> {
     let name = &pkg
         .manifest()
         .targets()
@@ -619,6 +646,7 @@ fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
     let mut version = pkg.version().clone();
     let mut install_subdir = None;
     let mut versioning = true;
+    let mut version_suffix_components = None;
     let mut import_library = true;
     let mut rustflags = Vec::new();
 
@@ -636,6 +664,19 @@ fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
             .get("versioning")
             .and_then(|v| v.as_bool())
             .unwrap_or(true);
+
+        if let Some(value) = library.get("version_suffix_components") {
+            let value = value.as_integer().with_context(|| {
+                format!("Value for `version_suffix_components` is not an integer: {value:?}")
+            })?;
+            version_suffix_components = Some(match value {
+                1 => VersionSuffix::Major,
+                2 => VersionSuffix::MajorMinor,
+                3 => VersionSuffix::MajorMinorPatch,
+                _ => anyhow::bail!("Out of range value for version suffix components: {value}"),
+            });
+        }
+
         import_library = library
             .get("import_library")
             .and_then(|v| v.as_bool())
@@ -650,11 +691,16 @@ fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
         }
     }
 
+    if rustc_target.os == "android" {
+        versioning = false;
+    }
+
     let library = LibraryCApiConfig {
         name: lib_name,
         version,
         install_subdir,
         versioning,
+        version_suffix_components,
         import_library,
         rustflags,
     };
@@ -754,14 +800,14 @@ fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
 
 fn compile_options(
     ws: &Workspace,
-    config: &Config,
+    gctx: &GlobalContext,
     args: &ArgMatches,
     profile: InternedString,
     compile_mode: CompileMode,
 ) -> anyhow::Result<CompileOptions> {
     use cargo::core::compiler::CompileKind;
     let mut compile_opts =
-        args.compile_options(config, compile_mode, Some(ws), ProfileChecking::Custom)?;
+        args.compile_options(gctx, compile_mode, Some(ws), ProfileChecking::Custom)?;
 
     compile_opts.build_config.requested_profile = profile;
 
@@ -779,12 +825,12 @@ fn compile_options(
 
     compile_opts.build_config.unit_graph = false;
 
-    let rustc = config.load_global_rustc(Some(ws))?;
+    let rustc = gctx.load_global_rustc(Some(ws))?;
 
     // Always set the target, requested_kinds is a vec of a single element.
     if compile_opts.build_config.requested_kinds[0].is_host() {
         compile_opts.build_config.requested_kinds =
-            CompileKind::from_requested_targets(config, &[rustc.host.to_string()])?
+            CompileKind::from_requested_targets(gctx, &[rustc.host.to_string()])?
     }
 
     Ok(compile_opts)
@@ -796,7 +842,6 @@ struct Exec {
     link_line: Mutex<HashMap<PackageId, String>>,
 }
 
-use cargo::core::*;
 use cargo::CargoResult;
 use cargo_util::ProcessBuilder;
 
@@ -846,7 +891,6 @@ impl Executor for Exec {
 
 use cargo::core::compiler::{unit_graph, UnitInterner};
 use cargo::ops::create_bcx;
-use cargo::util::profile;
 
 fn set_deps_args(
     dep: &UnitDep,
@@ -880,9 +924,9 @@ fn compile_with_exec(
 
     for unit in bcx.roots.iter() {
         let pkg = &unit.pkg;
-        let capi_config = load_manifest_capi_config(pkg)?;
+        let capi_config = load_manifest_capi_config(pkg, rustc_target)?;
         let name = &capi_config.library.name;
-        let install_paths = InstallPaths::new(name, args, &capi_config);
+        let install_paths = InstallPaths::new(name, rustc_target, args, &capi_config);
         let pkg_rustflags = &capi_config.library.rustflags;
 
         let mut leaf_args: Vec<String> = rustc_target
@@ -912,11 +956,10 @@ fn compile_with_exec(
     }
 
     if options.build_config.unit_graph {
-        unit_graph::emit_serialized_unit_graph(&bcx.roots, &bcx.unit_graph, ws.config())?;
+        unit_graph::emit_serialized_unit_graph(&bcx.roots, &bcx.unit_graph, ws.gctx())?;
         return Ok(HashMap::new());
     }
-    let _p = profile::start("compiling");
-    let cx = cargo::core::compiler::Context::new(&bcx)?;
+    let cx = cargo::core::compiler::BuildRunner::new(&bcx)?;
 
     let r = cx.compile(exec)?;
 
@@ -967,15 +1010,21 @@ impl CPackage {
         let id = pkg.package_id();
         let version = pkg.version().clone();
         let root_path = pkg.root().to_path_buf();
-        let capi_config = load_manifest_capi_config(pkg)?;
+        let capi_config = load_manifest_capi_config(pkg, rustc_target)?;
 
         patch_target(pkg, libkinds, &capi_config)?;
 
         let name = &capi_config.library.name;
 
-        let install_paths = InstallPaths::new(name, args, &capi_config);
-        let build_targets =
-            BuildTargets::new(name, rustc_target, root_output, libkinds, &capi_config)?;
+        let install_paths = InstallPaths::new(name, rustc_target, args, &capi_config);
+        let build_targets = BuildTargets::new(
+            name,
+            rustc_target,
+            root_output,
+            libkinds,
+            &capi_config,
+            args.get_flag("meson"),
+        )?;
 
         let finger_print = FingerPrint::new(&id, root_output, &build_targets, &install_paths);
 
@@ -992,21 +1041,21 @@ impl CPackage {
 
 pub fn cbuild(
     ws: &mut Workspace,
-    config: &Config,
+    config: &GlobalContext,
     args: &ArgMatches,
     default_profile: &str,
 ) -> anyhow::Result<(Vec<CPackage>, CompileOptions)> {
     let rustc = config.load_global_rustc(Some(ws))?;
-    let targets = args.targets();
-    let target = match targets.len() {
-        0 => rustc.host.to_string(),
-        1 => targets[0].to_string(),
+    let targets = args.targets()?;
+    let (target, is_target_overridden) = match targets.len() {
+        0 => (rustc.host.to_string(), false),
+        1 => (targets[0].to_string(), true),
         _ => {
             anyhow::bail!("Multiple targets not supported yet");
         }
     };
 
-    let rustc_target = target::Target::new(&target)?;
+    let rustc_target = target::Target::new(Some(&target), is_target_overridden)?;
 
     let default_kind = || match (rustc_target.os.as_str(), rustc_target.env.as_str()) {
         ("none", _) | (_, "musl") => vec!["staticlib"],
@@ -1019,7 +1068,7 @@ pub fn cbuild(
     let only_staticlib = !libkinds.contains(&"cdylib");
     let only_cdylib = !libkinds.contains(&"staticlib");
 
-    let profile = args.get_profile_name(config, default_profile, ProfileChecking::Custom)?;
+    let profile = args.get_profile_name(default_profile, ProfileChecking::Custom)?;
 
     let profiles = Profiles::new(ws, profile)?;
 
@@ -1109,7 +1158,7 @@ pub fn cbuild(
                     .unwrap()
                     .values()
                     .next()
-                    .unwrap()
+                    .map_or("", |s| s)
                     .to_string()
             };
             let capi_config = &cpkg.capi_config;
@@ -1164,6 +1213,7 @@ pub fn cbuild(
                     &root_output,
                     &libkinds,
                     capi_config,
+                    args.get_flag("meson"),
                 )?;
 
                 if let (Some(from_static_lib), Some(to_static_lib)) = (
@@ -1178,6 +1228,12 @@ pub fn cbuild(
                 ) {
                     copy(from_shared_lib, to_shared_lib)?;
                 }
+                if let (Some(from_debug_info), Some(to_debug_info)) = (
+                    from_build_targets.debug_info.as_ref(),
+                    build_targets.debug_info.as_ref(),
+                ) {
+                    copy(from_debug_info, to_debug_info)?;
+                }
             }
 
             cpkg.finger_print.static_libs = static_libs;
@@ -1186,6 +1242,11 @@ pub fn cbuild(
             // It is not a new build, recover the static_libs value from the cache
             cpkg.finger_print.static_libs = cpkg.finger_print.load_previous()?.static_libs;
         }
+
+        ws.gctx().shell().verbose(|s| {
+            let path = &format!("PKG_CONFIG_PATH=\"{}\"", root_output.display());
+            s.note(path)
+        })?;
     }
 
     Ok((members, compile_opts))
@@ -1193,13 +1254,12 @@ pub fn cbuild(
 
 pub fn ctest(
     ws: &Workspace,
-    config: &Config,
     args: &ArgMatches,
     packages: &[CPackage],
     mut compile_opts: CompileOptions,
 ) -> CliResult {
     compile_opts.build_config.requested_profile =
-        args.get_profile_name(config, "test", ProfileChecking::Custom)?;
+        args.get_profile_name("test", ProfileChecking::Custom)?;
     compile_opts.build_config.mode = CompileMode::Test;
 
     compile_opts.filter = ops::CompileFilter::new(
@@ -1246,3 +1306,66 @@ pub fn ctest(
 
     ops::run_tests(ws, &ops, &test_args)
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use semver::Version;
+
+    fn make_test_library_config(version: &str) -> LibraryCApiConfig {
+        LibraryCApiConfig {
+            name: "example".to_string(),
+            version: Version::parse(version).unwrap(),
+            install_subdir: None,
+            versioning: true,
+            version_suffix_components: None,
+            import_library: true,
+            rustflags: vec![],
+        }
+    }
+
+    #[test]
+    pub fn test_semver_zero_zero_zero() {
+        let library = make_test_library_config("0.0.0");
+        let sover = library.sover();
+        assert_eq!(sover, "0.0.0");
+    }
+
+    #[test]
+    pub fn test_semver_zero_one_zero() {
+        let library = make_test_library_config("0.1.0");
+        let sover = library.sover();
+        assert_eq!(sover, "0.1");
+    }
+
+    #[test]
+    pub fn test_semver_one_zero_zero() {
+        let library = make_test_library_config("1.0.0");
+        let sover = library.sover();
+        assert_eq!(sover, "1");
+    }
+
+    #[test]
+    pub fn text_one_fixed_zero_zero_zero() {
+        let mut library = make_test_library_config("0.0.0");
+        library.version_suffix_components = Some(VersionSuffix::Major);
+        let sover = library.sover();
+        assert_eq!(sover, "0");
+    }
+
+    #[test]
+    pub fn text_two_fixed_one_zero_zero() {
+        let mut library = make_test_library_config("1.0.0");
+        library.version_suffix_components = Some(VersionSuffix::MajorMinor);
+        let sover = library.sover();
+        assert_eq!(sover, "1.0");
+    }
+
+    #[test]
+    pub fn text_three_fixed_one_zero_zero() {
+        let mut library = make_test_library_config("1.0.0");
+        library.version_suffix_components = Some(VersionSuffix::MajorMinorPatch);
+        let sover = library.sover();
+        assert_eq!(sover, "1.0.0");
+    }
+}
